var sliceType = 1;
var rootType = 2;
var selectorType = 3;
var selectorAllType = 4;
var parentType = 5;
var firstChildType = 6;
var lastChildType = 7;
var prevSiblingType = 8;
var nextSiblingType = 9;
var removeType = 10;
var clearType = 11;
var htmlType = 12;
var replaceType = 13;
var appendType = 14;
var prependType = 15;
var insertAfterType = 16;
var insertBeforeType = 17;
var setAttrType = 18;
var replaceAttrType = 19;
var rmAttrType = 20;
var setStylesType = 21;
var rmStylesType = 22;
var addClassesType = 23;
var rmClassesType = 24;

function factory(type) {
  return function () {
    var result = [type];

    for (var i = 0; i < arguments.length; i++) {
      result.push(arguments[i]);
    }

    return result;
  };
}

var nil = [0];
var remove = [removeType];
var clear = [clearType];
var list = factory(sliceType);
var root = factory(rootType);
var one = factory(selectorType);
var all = factory(selectorAllType);
var parent = factory(parentType);
var firstChild = factory(firstChildType);
var lastChild = factory(lastChildType);
var prevSibling = factory(prevSiblingType);
var nextSibling = factory(nextSiblingType);
var html = factory(htmlType);
var replace = factory(replaceType);
var append = factory(appendType);
var prepend = factory(prependType);
var insertAfter = factory(insertAfterType);
var insertBefore = factory(insertBeforeType);
var setAttr = factory(setAttrType);
var replaceAttr = factory(replaceAttrType);
var rmAttr = factory(rmAttrType);
var setStyles = factory(setStylesType);
var rmStyles = factory(rmStylesType);
var addClasses = factory(addClassesType);
var rmClasses = factory(rmClassesType);

function mapNodes(nodes, mapFn) {
  var result = [];
  var children, i, j, c;

  for (i = 0; i < nodes.length; i++) {
    children = mapFn(nodes[i]);

    for (j = 0; j < children.length; j++) {
      c = children[j];

      if (c && c.nodeType == 1) {
        result.push(c);
      }
    }
  }

  return result;
}
function mapQuerySelector(selector) {
  return function (node) {
    return [node.querySelector(selector)];
  };
}
function mapQuerySelectorAll(selector) {
  return function (node) {
    return node.querySelectorAll(selector);
  };
}
function mapParent(node) {
  return [node.parentNode];
}
function mapFirstChild(node) {
  var i,
      n,
      cn = node.childNodes;

  for (i = 0; i < cn.length; i++) {
    n = cn[i];

    if (n.nodeType == 1) {
      return [n];
    }
  }

  return [];
}
function mapLastChild(node) {
  var i,
      n,
      cn = node.childNodes;

  for (i = cn.length - 1; i >= 0; i--) {
    n = cn[i];

    if (n.nodeType == 1) {
      return [n];
    }
  }

  return [];
}
function mapPrevSibling(node) {
  var ps = node.previousSibling;

  while (ps && ps.nodeType !== 1) {
    ps = ps.previousSibling;
  }

  return ps ? [ps] : [];
}
function mapNextSibling(node) {
  var ns = node.nextSibling;

  while (ns && ns.nodeType !== 1) {
    ns = ns.nextSibling;
  }

  return ns ? [ns] : [];
}

function fromClass(className) {
  var classMap = {};
  var classes = className.split(/[\s\uFEFF\xA0]+/);
  var i, c;

  for (i = 0; i < classes.length; i++) {
    c = classes[i];

    if (c) {
      classMap[c] = true;
    }
  }

  return classMap;
}
function toClass(classMap) {
  var classNames = [];
  var i;

  for (i in classMap) {
    if (classMap.hasOwnProperty(i) && classMap[i]) {
      classNames.push(i);
    }
  }

  return classNames.join(' ');
}

function setAttribute(node, key, value) {
  node.setAttribute(key, value);

  if (key == 'value' && node.tagName.toLowerCase() == 'input') {
    node.value = value;
  }
}
function forEach(elems, operation) {
  return new Promise(function (resolve) {
    var i = -1;

    function step() {
      i++;

      if (i < elems.length) {
        operation(elems[i]).then(step, step);
      } else {
        resolve();
      }
    }

    step();
  });
}

function clone(elems) {
  var result = [];

  for (var i = 0; i < elems.length; i++) {
    result.push(elems[i]);
  }

  return result;
}

function getFragment(node, html) {
  var fragment = node.cloneNode();
  fragment.innerHTML = html;
  markSingularNodes(fragment);
  return fragment;
}
function mapScript(node) {
  var isAttr = node.getAttribute('is');
  var document = node.ownerDocument;
  var script;

  if (isAttr) {
    script = document.createElement('script', {
      is: isAttr
    });
  } else {
    script = document.createElement('script');
  }

  script.text = node.text;

  for (var i = 0; i < node.attributes.length; i++) {
    var attribute = node.attributes[i];
    script.setAttribute(attribute.name, attribute.value);
  }

  return script;
}

function onLoad(node) {
  return new Promise(function (resolve) {
    if (!('onload' in node)) {
      resolve();
      return;
    }

    if (node.addEventListener) {
      node.addEventListener('load', resolve, false);
      node.addEventListener('error', resolve, false);
      return;
    }

    if (node.attachEvent) {
      node.attachEvent('onload', resolve);
      node.attachEvent('onerror', resolve);
      node.attachEvent('onreadystatechange', function () {
        if (node.readyState == 'loaded' || node.readyState == 'complete') {
          resolve();
        }
      });
      return;
    }

    resolve();
  });
}

function markSingularNodes(fragment) {
  var singularNodes = fragment.querySelectorAll('script, link[rel=stylesheet]');

  for (var i = 0; i < singularNodes.length; i++) {
    var node = singularNodes[i];

    while (node) {
      node.__witSingularNode = true;
      node = node.parentNode;
    }
  }
}

function waitUntilScriptLoaded(node) {
  if (node.src) {
    return onLoad(node);
  }

  return Promise.resolve();
}

function waitUntilLinkLoaded(node) {
  if (node.href && node.rel == 'stylesheet') {
    return onLoad(node);
  }

  return Promise.resolve();
}

function transfer(trf, parent) {
  return forEach(clone(parent.childNodes), function (node) {
    if (!node.__witSingularNode) {
      trf(node);
      return Promise.resolve();
    }

    switch (node.tagName.toLowerCase()) {
      case 'script':
        var script = mapScript(node);
        trf(script);
        return waitUntilScriptLoaded(script);

      case 'link':
        trf(node);
        return waitUntilLinkLoaded(node);

      default:
        var _clone = node.cloneNode();

        trf(_clone);
        return transfer(function (child) {
          return _clone.appendChild(child);
        }, node);
    }
  });
}

function apply(delta, rootNode, nodes) {
  switch (delta[0]) {
    case sliceType:
      return forEach(delta.slice(1), function (d) {
        return apply(d, rootNode, nodes);
      });

    case rootType:
      return forEach(delta.slice(1), function (d) {
        return apply(d, rootNode, [rootNode]);
      });

    case selectorType:
      return forEach(delta.slice(2), function (d) {
        return apply(d, rootNode, mapNodes(nodes, mapQuerySelector(delta[1])));
      });

    case selectorAllType:
      return forEach(delta.slice(2), function (d) {
        return apply(d, rootNode, mapNodes(nodes, mapQuerySelectorAll(delta[1])));
      });

    case parentType:
      return forEach(delta.slice(1), function (d) {
        return apply(d, rootNode, mapNodes(nodes, mapParent));
      });

    case firstChildType:
      return forEach(delta.slice(1), function (d) {
        return apply(d, rootNode, mapNodes(nodes, mapFirstChild));
      });

    case lastChildType:
      return forEach(delta.slice(1), function (d) {
        return apply(d, rootNode, mapNodes(nodes, mapLastChild));
      });

    case prevSiblingType:
      return forEach(delta.slice(1), function (d) {
        return apply(d, rootNode, mapNodes(nodes, mapPrevSibling));
      });

    case nextSiblingType:
      return forEach(delta.slice(1), function (d) {
        return apply(d, rootNode, mapNodes(nodes, mapNextSibling));
      });

    case removeType:
      {
        for (var i = 0; i < nodes.length; i++) {
          var node = nodes[i];

          if (node.parentNode) {
            node.parentNode.removeChild(node);
          }
        }

        break;
      }

    case clearType:
      {
        for (var _i = 0; _i < nodes.length; _i++) {
          nodes[_i].innerHTML = '';
        }

        break;
      }

    case setAttrType:
      {
        var attrMap = delta[1];

        for (var _i2 = 0; _i2 < nodes.length; _i2++) {
          var _node = nodes[_i2];

          for (var j in attrMap) {
            if (attrMap.hasOwnProperty(j)) {
              setAttribute(_node, j, attrMap[j]);
            }
          }
        }

        break;
      }

    case replaceAttrType:
      {
        var _attrMap = delta[1];

        for (var _i3 = 0; _i3 < nodes.length; _i3++) {
          var _node2 = nodes[_i3];

          for (var _j = _node2.attributes.length - 1; _j >= 0; _j--) {
            var name = _node2.attributes[_j].name;

            if (!_attrMap.hasOwnProperty(name)) {
              _node2.removeAttribute(name);
            }
          }

          for (var _j2 in _attrMap) {
            if (_attrMap.hasOwnProperty(_j2)) {
              setAttribute(_node2, _j2, _attrMap[_j2]);
            }
          }
        }

        break;
      }

    case rmAttrType:
      {
        for (var _i4 = 0; _i4 < nodes.length; _i4++) {
          var _node3 = nodes[_i4];

          for (var _j3 = 1; _j3 < delta.length; _j3++) {
            _node3.removeAttribute(delta[_j3]);
          }
        }

        break;
      }

    case setStylesType:
      {
        var stylesMap = delta[1];

        for (var _i5 = 0; _i5 < nodes.length; _i5++) {
          var _node4 = nodes[_i5];

          for (var _j4 in stylesMap) {
            if (stylesMap.hasOwnProperty(_j4)) {
              var m = stylesMap[_j4].match(/^(.*?)(\s*)!important$/);

              if (m) {
                _node4.style.setProperty(_j4, m[1], 'important');
              } else {
                _node4.style.setProperty(_j4, stylesMap[_j4]);
              }
            }
          }
        }

        break;
      }

    case rmStylesType:
      {
        for (var _i6 = 0; _i6 < nodes.length; _i6++) {
          var _node5 = nodes[_i6];

          for (var _j5 = 1; _j5 < delta.length; _j5++) {
            _node5.style.removeProperty(delta[_j5]);
          }
        }

        break;
      }

    case addClassesType:
      {
        var _stylesMap = fromClass(delta[1]);

        for (var _i7 = 0; _i7 < nodes.length; _i7++) {
          var _node6 = nodes[_i7];
          var classMap = fromClass(_node6.className);

          for (var _j6 in _stylesMap) {
            if (_stylesMap.hasOwnProperty(_j6)) {
              classMap[_j6] = true;
            }
          }

          _node6.className = toClass(classMap);
        }

        break;
      }

    case rmClassesType:
      {
        var _classMap = fromClass(delta[1]);

        for (var _i8 = 0; _i8 < nodes.length; _i8++) {
          var _node7 = nodes[_i8];
          var nodeClassMap = fromClass(_node7.className);

          for (var _j7 in _classMap) {
            if (_classMap.hasOwnProperty(_j7)) {
              delete nodeClassMap[_j7];
            }
          }

          _node7.className = toClass(nodeClassMap);
        }

        break;
      }

    case htmlType:
      return forEach(nodes, function (node) {
        node.innerHTML = '';
        return transfer(function (child) {
          return node.appendChild(child);
        }, getFragment(node, delta[1]));
      });

    case replaceType:
      return forEach(nodes, function (node) {
        var parent = node.parentNode;

        if (!parent) {
          return Promise.resolve();
        }

        var ref = node.nextSibling;
        parent.removeChild(node);
        return transfer(function (child) {
          return parent.insertBefore(child, ref);
        }, getFragment(parent, delta[1]));
      });

    case appendType:
      return forEach(nodes, function (node) {
        return transfer(function (child) {
          return node.appendChild(child);
        }, getFragment(node, delta[1]));
      });

    case prependType:
      return forEach(nodes, function (node) {
        var firstChild = node.firstChild;
        return transfer(function (child) {
          return node.insertBefore(child, firstChild);
        }, getFragment(node, delta[1]));
      });

    case insertAfterType:
      return forEach(nodes, function (node) {
        if (!node.parentNode) {
          return Promise.resolve();
        }

        return transfer(function (child) {
          return node.parentNode.insertBefore(child, node.nextSibling);
        }, getFragment(node.parentNode, delta[1]));
      });

    case insertBeforeType:
      return forEach(nodes, function (node) {
        if (!node.parentNode) {
          return Promise.resolve();
        }

        return transfer(function (child) {
          return node.parentNode.insertBefore(child, node);
        }, getFragment(node.parentNode, delta[1]));
      });
  }

  return Promise.resolve();
}

var mutex = window.witMutex = window.witMutex || function () {
  var queued = [];
  var locked = true;

  var unlockQueue = function unlockQueue() {
    if (queued.length > 0) {
      queued.shift()();
    } else {
      locked = false;
    }
  };

  var mutex = {
    lock: function lock() {
      return new Promise(function (resolve) {
        var _unlock = function unlock() {
          _unlock = function unlock() {};

          unlockQueue();
        };

        if (!locked) {
          locked = true;
          resolve(function () {
            return _unlock();
          });
          return;
        }

        queued.push(function () {
          return resolve(function () {
            return _unlock();
          });
        });
      });
    }
  };

  var _cb = function cb() {
    if (window.removeEventListener) {
      window.removeEventListener('load', _cb, false);
      document.removeEventListener('DOMContentLoaded', _cb, false);
    } else if (window.detachEvent) {
      window.detachEvent('onload', _cb);
    }

    _cb = function cb() {};

    unlockQueue();
  };

  if (document.readyState == 'complete' || document.readyState == 'interactive') {
    unlockQueue();
  } else if (window.addEventListener) {
    window.addEventListener('load', _cb, false);
    document.addEventListener('DOMContentLoaded', _cb, false);
  } else if (window.attachEvent) {
    window.attachEvent('onload', _cb);
  }

  return mutex;
}();

var index = (function (delta) {
  var rootNode, nodes;

  for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  for (var _i2 = 0; _i2 < args.length; _i2++) {
    var arg = args[_i2];

    if (arg instanceof Array) {
      nodes = arg;
    } else {
      rootNode = arg;
    }
  }

  rootNode = rootNode || document.documentElement;
  nodes = nodes || [rootNode];
  return mutex.lock().then(function (unlock) {
    return apply(delta, rootNode, nodes).then(unlock, unlock);
  });
});

export { addClasses, all, append, index as apply, clear, firstChild, html, insertAfter, insertBefore, lastChild, list, mutex, nextSibling, nil, one, parent, prepend, prevSibling, remove, replace, replaceAttr, rmAttr, rmClasses, rmStyles, root, setAttr, setStyles };
